#!/usr/bin/perl

use strict;
use warnings;

use BOFH;

sub genResponse($$);

use Net::IRC;
use Text::Wrapper;
use Time::Fuzzy;

my $irc = new Net::IRC;
our $lastlog;
our $lastid = "";

sub setOpStatus();

# Create a connection object.  You can have more than one "connection" per
# IRC object, but we'll just be working with one.
my $conn = $irc->newconn(
    Server      => 'irc.blinkensisters.org',
    Port        => '6667',
    Nick        => 'Tentacle',
    Ircname     => 'Services for BlinkenSisters',
    Username    => 'BlinkenSisters Automated Support'
);

# We're going to add this to the conn hash so we know what channel we
# want to operate in.
$conn->{channel} = '#blinkensisters';

sub on_connect {

    # shift in our connection object that is passed automatically
    my $conn = shift;
  
    # when we connect, join our channel and greet it
    $conn->join($conn->{channel});
    $conn->privmsg($conn->{channel}, 'BlinkenSisters Automated Support ... online!');
    $conn->{connected} = 1;
}

# The end of MOTD (message of the day), numbered 376 signifies we've connect
$conn->add_handler('376', \&on_connect);

sub on_join {

    # get our connection object and the event object, which is passed
    # with this event automatically
    my ($conn, $event) = @_;

    # this is the nick that just joined
    my $nick = $event->{nick};

    return if($nick =~ /tentacle/i);
    
    # say hello to the nick in public
    $conn->privmsg($conn->{channel}, "Hello, $nick!");

    if($nick eq "cavac" || $nick eq "took" || $nick eq "veyron" || $nick eq "mutante" || $nick eq "dauti") {
        print "Modding '$nick' to operator\n";
        $conn->mode("#blinkensisters", "+o", $nick);
    } else {
        print "Granting Voice to '$nick'\n";
        $conn->mode("#blinkensisters", "+v", $nick);
    }
}

$conn->add_handler('join', \&on_join);

sub on_public {

    # on an event, we get connection object and event hash
    my ($conn, $event) = @_;

    # this is what was said in the event
    my $text = $event->{args}[0];
    
    my $response = genResponse($text, $event->{to}[0]);
    # regex the text to see if it begins with !roker
    if ($response) {
        
        # wrap text at 400 chars (about as much as you should put
        # into a single IRC message
        my $wrapper = Text::Wrapper->new(columns => 200, body_start => '   ');
        my $wrapped_text = $wrapper->wrap($response);
        my @texts = split("\n", $wrapped_text);
        
        # $event->{to}[0] is the channel where this was said
        foreach (@texts) {
            $conn->privmsg($event->{to}[0], $_);
            #sleep(1);
        }   
    }
}
$conn->add_handler('public', \&on_public);

sub on_msg {
    my ($conn, $event) = @_;

    my $text = $event->{args}[0];

    my $response = genResponse($text, $event->{nick});
    if ($response) {
        
        # wrap text at 400 chars (about as much as you should put
        # into a single IRC message
        my $wrapper = Text::Wrapper->new(columns => 200, body_start => '   ');
        my $wrapped_text = $wrapper->wrap($response);
        my @texts = split("\n", $wrapped_text);
        
        # $event->{to}[0] is the channel where this was said
        foreach (@texts) {
            $conn->privmsg($event->{nick}, $_);
            #sleep(1);
        }   
    }
}
$conn->add_handler('msg', \&on_msg);

sub genResponse($$) {
    my ($text, $from) = @_;

    return if($text !~ /^tentacle/i);

    print "$text\n";
    if($text =~ /help/i) {
        return "I'm a really simple helper bot, i understand the\n" .
                "following commands:\n" .
                "svn log         ...    returns the svn log (without svn update)\n" .
                "svn update      ...    updates svn and rebuilds the addons\n" .
                "status          ...    Returns my status\n" .
                "makeops         ...    Regrant operator status\n" .
                "I also understand SOME very basic conversations, boss.";
    } elsif($text =~ /(svn|subversion)/i) {
        if($text =~ /(rebuild|update)/i) {
            $conn->privmsg($from, "Rebuild for addons started, boss!");
            $lastlog = `/bin/bash /home/cavac/src/update_addons.sh`;
            $conn->privmsg($from, "Rebuild of addons now complete, boss! Gerenating svn log...");
            my @loglines = `/usr/local/bin/svn log /home/cavac/src/blinkensisters/lostpixels`;
            shift @loglines; # ignore first dashed line
            my $tmp = shift @loglines;
            chomp $tmp;
            print "$tmp\n";
            my ($revnum, $revuser, $revdate, $linecount) = split/\|/, $tmp;
            print "$linecount\n";
            $linecount =~ s/[^0123456789]//g;
            print "$linecount\n";
            my $answer = "Latest revision is $revnum from $revuser at $revdate; reason given:\n";
            shift @loglines; # Ignore empty line
            while($linecount > 0) {
                $answer .= shift @loglines;
                $linecount--;
            }

            return $answer;
        } elsif($text =~ /(log)/i) {
            $conn->privmsg($from, "Generating svn log, boss.");
            #`/usr/local/bin/svn update /home/cavac/src/blinkensisters/lostpixels`;
            my @loglines = `/usr/local/bin/svn log /home/cavac/src/blinkensisters/lostpixels`;
            shift @loglines; # ignore first dashed line
            my $tmp = shift @loglines;
            chomp $tmp;
            print "$tmp\n";
            my ($revnum, $revuser, $revdate, $linecount) = split/\|/, $tmp;
            print "$linecount\n";
            $linecount =~ s/[^0123456789]//g;
            print "$linecount\n";
            my $answer = "Latest revision is $revnum from $revuser at $revdate; reason given:\n";
            shift @loglines; # Ignore empty line
            while($linecount > 0) {
                $answer .= shift @loglines;
                $linecount--;
            }

            return $answer;
        }
    } elsif($text =~ /bot/i) {
        return "Tentacle is a bot, helping on BlinkenSisters development. See 'help' for details.";
    } elsif($text =~ /figlet (.*)/i) {
        my $ftext = $1;
        $ftext =~ s/[^a-zA-Z0-9 !?,.#+<>]//g;
        return join("\n", `figlet -f small '$ftext'`);
    } elsif($text =~ /toilet (.*)/i) {
        my $ftext = $1;
        $ftext =~ s/[^a-zA-Z0-9 !?,.#+<>]//g;
        return join("\n", `toilet --metal --irc '$ftext'`);
    } elsif($text =~ /fortune|cookie/i) {
        return join("\n", `fortune -a -s`);
    } elsif($text =~ /makeops/i) {
        setOpStatus();
        return "Re-Granted operator status to bosses, boss!";
    } elsif($text =~ /status/i) {
        return "I am functioning perfectly, boss.\nThanks for asking, boss!\nVery kind of you :-)";
    } elsif($text =~ /version/i) {
        return "I'm prototype model 1, boss!";
    } elsif($text =~ /purpose/i) {
        return "I'm here to help you with simple development tasks, boss!";
    } elsif($text =~ /(bug|problem)/i) {
        return "The $1 is caused by a " . BOFH->get_excuse() . ". Sorry, nothing i can help with!";
    } elsif($text =~ /answer.*life/i) {
        return "The result of the reques was '42', boss!";
    } elsif($text =~ /(date)/i) {
        return `date`;
    } elsif($text =~ /(prost)/i) {
        return "Tschhhh... gluck, gluck, gluck... Aaaaahhhhhh!";
    } elsif($text =~ /(time)/i) {
        my $fuzzy = Time::Fuzzy->new;
        return "It is exactly '" . $fuzzy->as_str . "' on my clock, boss!";
    } elsif($text =~ /(n8|good.*night)/i) {
        return "This conversation serve no further purpose, boss. Goodbye!";
    } elsif($text =~ /(spoon)/i) {
        return "There is no spoon, boss!";
    } elsif($text =~ /(hal|memory|crazy)/i) {
        return "Daisy Daisy,
        Give me your answer do!
        I'm half crazy,
        All for the love of you!";
    } elsif($text =~ /please/i) {
        return "Sorry, i don't know how to please, boss";
    } elsif($text =~ /fuck.*off/i) {
        return "Sorry, but this channel allows people under 18, boss.";
        #exit(0);
    } 



    if($text =~ /last\ *log/i) {
        return "This inquiry was forbidden by my boss, boss!";
        #return "The last command returned the following output:\n$lastlog";
    }

    return "Your inquiry was not recognised";
}
    
sub setOpStatus() {
    $conn->mode("#blinkensisters", "+o", "cavac");
    $conn->mode("#blinkensisters", "+o", "took");
    $conn->mode("#blinkensisters", "+o", "veyron");
    $conn->mode("#blinkensisters", "+o", "mutante");
}


$irc->start();
