// BlinkenSisters - Hunt for the Lost Pixels
//     Bringing back the fun of the 80s
//
// (C) 2005-07 Rene Schickbauer, Wolfgang Dautermann
//
// See License.txt for licensing information
//


#include "globals.h"
#include "playersprite.h"
#include "memops.h"
#include "errorhandler.h"
#include "engine.h"
#include "convert.h"

SDL_Surface *SPRITE_Left;
SDL_Surface *SPRITE_Right;
SDL_Surface* sprite;
MOVE_TYPE lastmove = NO_MOVE;
Uint32 spritenum = 0;
Uint32 spritelasttick;
Uint32 spritetick;
Uint32 sprite_srcoffset = 0;
Uint32 playersprite_srcoffset;

void initPlayerSprite() {
	char fullfname[MAX_FNAME_LENGTH];
	sprintf(fullfname, "%s", configGetPath("sister_moveleft.bmp"));
	SDL_Surface* temp = IMG_Load(fullfname);
	if(!temp) {
		DIE(ERROR_IMAGE_READ, fullfname);
	}
	SPRITE_Left = convertToBSSurface(temp);
	SDL_FreeSurface(temp);

	sprintf(fullfname, "%s", configGetPath("sister_moveright.bmp"));
	temp = IMG_Load(fullfname);
	if(!temp) {
		DIE(ERROR_IMAGE_READ, fullfname);
	}
	SPRITE_Right = convertToBSSurface(temp);
	SDL_FreeSurface(temp);
	
	sprite = SPRITE_Left;
	
	spritetick = spritelasttick = BS_GetTicks();
	
}


void deInitPlayerSprite() {
	SDL_FreeSurface(SPRITE_Left);
	SDL_FreeSurface(SPRITE_Right);
}

void blitSprite(const Uint32 x, const Uint32 y, SDL_Surface* srcSurface, const Uint32 num) {
    static SDL_Rect src;
	static SDL_Rect dest;

    src.x = 0;
    src.y = TILESIZE * num;
    src.w = dest.w = src.h = dest.h = TILESIZE;
    dest.x = x;
    dest.y = y;
    
	SDL_BlitSurface(srcSurface, &src, gScreen, &dest);
}

void showSprite(const Uint32 x, const Uint32 y, const MOVE_TYPE mtype) {
	spritetick = BS_GetTicks();
	switch (mtype) {
		case MOVE_LEFT:
			sprite = SPRITE_Left;
			break;
		case MOVE_RIGHT:
			sprite = SPRITE_Right;
			break;
		case JUMP_LEFT:
			break;
		case JUMP_RIGHT:
			break;
		default:
			break;
	}
	if(lastmove != mtype || freeMovement || mtype == NO_MOVE) {
		spritenum = 0;
		if(!turboMode) {
			spritelasttick = spritetick + SPRITE_SWITCHWAIT;
		} else {
			spritelasttick = spritetick + SPRITE_SWITCHWAIT / 4;
		}
	} else {
		
		if(spritelasttick < spritetick) {
			spritenum++;
			if(spritenum >= (Uint32)(sprite->h / TILESIZE)) {
				spritenum = 0;
			}
			if(!turboMode) {
				spritelasttick = spritetick + SPRITE_SWITCHWAIT;
			} else {
				spritelasttick = spritetick + SPRITE_SWITCHWAIT / 4;
			}
		}
	}
	playersprite_srcoffset = spritenum * TILESIZE;
	blitSprite(x, y, sprite, spritenum);	
	
	lastmove = mtype;
	
}

